package at.ac.uibk.gitsearch.repository.jpa;

import at.ac.uibk.gitsearch.domain.User;
import at.ac.uibk.gitsearch.domain.UserWatchList;
import java.util.List;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

/**
 * Spring Data  repository for the UserWatchList entity.
 */
@Repository
public interface UserWatchListRepository extends JpaRepository<UserWatchList, Long>, JpaSpecificationExecutor<UserWatchList> {
    @Query("select userWatchList from UserWatchList userWatchList where userWatchList.user.login = ?#{principal.username}")
    List<UserWatchList> findByUserIsCurrentUser();

    @Query("select uwl from UserWatchList uwl where uwl.name LIKE CONCAT('%',:query,'%')")
    Page<UserWatchList> search(@Param("query") String query, Pageable pageable);

    /**
     * Returns all watchlists of the current user.
     *
     * @param user the user
     * @return the list of watchlists
     */
    List<UserWatchList> findByUser(User user);

    @Query("select uwl from UserWatchList uwl where uwl.user.login = ?#{principal.username} AND uwl.name LIKE CONCAT('%',:query,'%')")
    Page<UserWatchList> searchForCurrentUser(@Param("query") String query, Pageable pageable);
}
