import { Component, OnInit } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';

import { IUserWatchList } from '../user-watch-list.model';
import { UserWatchListService } from '../service/user-watch-list.service';
import { UserWatchListDeleteDialogComponent } from '../delete/user-watch-list-delete-dialog.component';

@Component({
  selector: 'jhi-user-watch-list',
  templateUrl: './user-watch-list.component.html',
})
export class UserWatchListComponent implements OnInit {
  userWatchLists?: IUserWatchList[];
  isLoading = false;

  constructor(protected userWatchListService: UserWatchListService, protected modalService: NgbModal) {}

  loadAll(): void {
    this.isLoading = true;

    this.userWatchListService.query().subscribe({
      next: (res: HttpResponse<IUserWatchList[]>) => {
        this.isLoading = false;
        this.userWatchLists = res.body ?? [];
      },
      error: () => {
        this.isLoading = false;
      },
    });
  }

  ngOnInit(): void {
    this.loadAll();
  }

  trackId(index: number, item: IUserWatchList): number {
    return item.id!;
  }

  delete(userWatchList: IUserWatchList): void {
    const modalRef = this.modalService.open(UserWatchListDeleteDialogComponent, { size: 'lg', backdrop: 'static' });
    modalRef.componentInstance.userWatchList = userWatchList;
    // unsubscribe not needed because closed completes on modal close
    modalRef.closed.subscribe(reason => {
      if (reason === 'deleted') {
        this.loadAll();
      }
    });
  }
}
