package at.ac.uibk.gitsearch.service;

import at.ac.uibk.gitsearch.GitsearchApp;
import at.ac.uibk.gitsearch.repository.search.testESService.ElasticSearchTestConfiguration;
import at.ac.uibk.gitsearch.service.ShoppingBasketService.ShoppingBasketInfoDTO;
import at.ac.uibk.gitsearch.service.ShoppingBasketService.ShoppingBasketRedirectInfoDTO;
import java.io.IOException;
import java.util.concurrent.TimeUnit;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.codeability.sharing.plugins.api.SharingPluginConfig;
import org.codeability.sharing.plugins.api.SharingPluginConfig.Action;
import org.codeability.sharing.plugins.api.ShoppingBasket;
import org.codeability.sharing.plugins.api.search.SearchInputDTO;
import org.codeability.sharing.plugins.api.search.SearchInputMetadataDTO;
import org.codeability.sharing.plugins.api.search.SearchResultDTO;
import org.codeability.sharing.plugins.api.search.SearchResultsDTO;
import org.elasticsearch.node.NodeValidationException;
import org.junit.Assert;
import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.Timeout;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.security.test.context.support.WithMockUser;

@SpringBootTest(classes = GitsearchApp.class)
@WithMockUser(value = "test", authorities = "sharing")
public class ShoppingBasketServiceIT {

    @Autowired
    protected ShoppingBasketService shoppingBasketService;

    @Autowired
    private SearchService searchService;

    @SuppressWarnings("unused")
    private static final Logger log = LoggerFactory.getLogger(ShoppingBasketServiceIT.class);

    @Autowired
    ElasticSearchTestConfiguration elasticSearchTestConfiguration;

    @Autowired
    @SuppressWarnings("PMD.ImmutableField")
    private PluginManagementService pluginManagementService;

    static ElasticSearchTestConfiguration staticElasticSearchTestConfiguration;

    @BeforeEach // must be started as BeforeEach, in order to autowire the ElasticSearchTestConfiguration
    @Timeout(value = 2, unit = TimeUnit.MINUTES)
    public void setUpESServer() throws IOException, NodeValidationException {
        if (staticElasticSearchTestConfiguration == null) {
            elasticSearchTestConfiguration.startTestNode();
            staticElasticSearchTestConfiguration = elasticSearchTestConfiguration;
        }
    }

    @AfterAll
    public static void shutDownESServer() {
        if (staticElasticSearchTestConfiguration != null) {
            staticElasticSearchTestConfiguration.stopTestContainer();
            staticElasticSearchTestConfiguration = null;
        }
    }

    @Test
    public void testGetBasket() throws IOException {
        // setup Plugin info
        SharingPluginConfig pluginInfo = new SharingPluginConfig(
            "testPlugin",
            new Action[] { new Action("testActionId", "http://localhost/actionTransfer", "testAction", "unused FilterExpression") }
        );

        pluginManagementService.addPlugin(pluginInfo, "http://localhost/testPlugin");
        // just load everything from index :-)
        final SearchInputMetadataDTO searchMetadata = new SearchInputMetadataDTO(null, null, null, null, null, null);
        SearchInputDTO searchQuery = new SearchInputDTO(null, searchMetadata, null, null, null, 0);
        SearchResultsDTO searchResultPage = searchService.searchResultPage(searchQuery);
        Assert.assertTrue("At least one hit?", searchResultPage.getSearchResult().size() >= 1);

        ShoppingBasketInfoDTO sbi = new ShoppingBasketInfoDTO(searchResultPage.getSearchResult().toArray(new SearchResultDTO[] {}));
        sbi.setPlugin(pluginInfo.pluginName);
        sbi.setAction(pluginInfo.actions[0].actionId);

        ShoppingBasketRedirectInfoDTO redInfo = shoppingBasketService.getRedirectInfo(sbi, "http://unused/unused");

        Pattern tokenParser = Pattern.compile("/([^/?]*)\\?");
        final Matcher matcher = tokenParser.matcher(redInfo.getRedirectURL());
        Assert.assertTrue(matcher.find());
        String token = matcher.group(1);
        final ShoppingBasket basket = shoppingBasketService.getBasket(token);

        int count = 0;
        for (SearchResultDTO hitResult : searchResultPage.getSearchResult()) {
            // we assume same sequence
            Assert.assertEquals(basket.exerciseInfo[count].getMetadata().getTitle(), hitResult.getMetadata().getTitle());
            count++;
        }
    }
}
