package at.ac.uibk.ac.artemis;

public class Receiver implements LeakyBucketPatternInterface {

	/**
	 * the error rate per second.
	 */
	private float errorRatePerSec;
	
	/**
	 * internal parameter to hold the current number of errors in bucket.
	 */
	private double currentErrorRate = 0.0f;
	/**
	 * timestamp for the last error received.
	 */
	private long lastErrorReceived = 0L;
	
	@Override
	public void initReceiver(float errorRatePerSec) {
		this.errorRatePerSec = errorRatePerSec;
	}

	@Override
	public void receivePacket(PotentiallyFaultyPacket potentiallyFaultyPacket) throws TooManyErrorsException {
		if(!potentiallyFaultyPacket.isFaulty()) {
			// just proceed
			return;
		}
		// we received a faulty package :-(
		// calculate current bucket fill
		long now = System.currentTimeMillis();
		long timeSinceLastError = now-lastErrorReceived;
		lastErrorReceived = now;
		
		currentErrorRate = currentErrorRate - (timeSinceLastError)/1000.0f;
		if(currentErrorRate < 0) currentErrorRate = 0.0f;
		currentErrorRate = currentErrorRate + 1.0;
		
		System.out.println("Time: " + now + "/" + timeSinceLastError + ": " + currentErrorRate + " [max: "+  errorRatePerSec+ "]");
		if(currentErrorRate > errorRatePerSec) throw new TooManyErrorsException();
		
	}

}
